#!/usr/bin/env perl

use warnings;
use strict;
use utf8;
use feature ":5.10";

use Data::Dumper;
use Hash::Merge qw( merge );
use List::MoreUtils qw(uniq);
use Params::Util qw(_HASH);

Hash::Merge::specify_behavior(
    {
        SCALAR => {
            SCALAR => sub { [ uniq $_[0], $_[1] ] },
            ARRAY  => sub {
                my ($scalar, $array) = @_;
                my %keys = map { $_ => 1 } map { keys %$_ } grep { _HASH $_ } @$array;
                [ uniq( ( $keys{$scalar} ? () : $scalar ), @$array ) ];
            },
            HASH   => sub {
                my ($scalar, $hash) = @_;
                my %keys = map { $_ => 1 } keys %$hash;
                [ ( $keys{$scalar} ? () : $scalar ), $hash ];
            },
        },
        ARRAY => {
            SCALAR => sub { 
                my ($array, $scalar) = @_;
                my %keys = map { $_ => 1 } map { keys %$_ } grep { _HASH $_ } @$array;
                [ uniq( ( $keys{$scalar} ? () : $scalar ), @$array ) ];
            },
            ARRAY  => sub { [ @{$_[0]}, @{$_[1]} ] },
            HASH   => sub { 
                my ($array, $hash) = @_;
                my @s;
                my @h;
                for (@$array) {
                    if (_HASH $_) {
                        push @h, $_;
                    }
                    else {
                        push @s, $_;
                    }
                }
                $hash = Hash::Merge::_merge_hashes($hash, $_) for @h;
                [ (map { $hash->{$_} ? () : $_ } @s), $hash ];
            },
        },
        HASH => {
            SCALAR => sub {
                my ($hash, $scalar) = @_;
                my %keys = map { $_ => 1 } keys %$hash;
                [ ( $keys{$scalar} ? () : $scalar ), $hash ];
            },
            ARRAY  => sub {
                my ($hash, $array) = @_;
                my @s;
                my @h;
                for (@$array) {
                    if (_HASH $_) {
                        push @h, $_;
                    }
                    else {
                        push @s, $_;
                    }
                }
                $hash = Hash::Merge::_merge_hashes($hash, $_) for @h;
                [ (map { $hash->{$_} ? () : $_ } @s), $hash ];
            },
            HASH   => sub { Hash::Merge::_merge_hashes( $_[0], $_[1] ) }, 
        },
    }, 
);

my $a = { a => 'b' };
my $b = { a => 'c' };
my $c = merge( $a, $b );
warn Dumper $c;
my $d = { a => { b => 'c' } };
my $e = merge( $c, $d );
warn Dumper $e;
my $f = { a => { d => 'c' } };
my $g = merge( $e, $f );
warn Dumper $g;
warn Dumper $b;
warn Dumper merge( $b, $g );
warn Dumper merge( merge( $b, $g ), $a );
