use lib "xt/lib";
use Modern::Perl;
use Test::More;
use Data::Dumper;
use MyTest1::Schema;
use Search::Query;

use_ok('Search::Query::Dialect::DBIxClass::Fuzzy');

# my $schema = do {
#     my $DB_FILE = "tmp.sqlite";
#     system( "rm $DB_FILE && sqlite3 $DB_FILE < schema.sql" ) == 0 or die;
#     MyTest1::Schema->connect("dbi:SQLite:dbname=$DB_FILE");
# };
my $schema = MyTest1::Schema->connect("dbi:Pg:dbname=$ENV{DBNAME}", $ENV{DBUSER}, $ENV{DBPASS});

{
    my $rs = $schema->resultset('Artist');
    $rs->delete;
    $rs->create($_) for (
        {
            name => 'Dream Theater',
            albums => [
                {
                    name => 'Images And Words',
                    tracks => [
                        { name => 'Another Day'    , } ,
                        { name => 'Wait For Sleep' , } ,
                    ],
                },
                {
                    name => 'Awake',
                    tracks => [
                        { name => 'Innocence Faded'             , } ,
                        { name => 'Lifting Shadows Off A Dream' , } ,
                    ],
                },
            ],
        },
        {
            name => 'Liquid Tension Experiment',
            albums => [
                {
                    name => 'Liquid Tension Experiment 2',
                    tracks => [
                        { name => 'Acid Rain' , } ,
                    ],
                },
            ],
        },
    );
}

{
    my $rs = $schema->resultset('Artist');
    my $parser = Search::Query->parser(
        default_field => [qw(id name)],
        dialect => 'DBIxClass::Fuzzy',
        query_class_opts => {
            result_source => $rs->result_source,
        },
    );
    my $query = $parser->parse("Dream Theater");
    $rs = $rs->search($query->as_dbic_query);
    my @items = $rs->hashref_array;
    is @items, 1;
    is $items[0]{name}, 'Dream Theater';
}

{
    my $rs = $schema->resultset('Artist');
    my $parser = Search::Query->parser(
        default_field => [qw(id name)],
        dialect => 'DBIxClass::Fuzzy',
        query_class_opts => {
            result_source => $rs->result_source,
        },
    );
    my $query = $parser->parse("Dream Theater");
    $rs = $rs->search($query->as_dbic_query);
    my @items = $rs->hashref_array;
    is @items, 1;
    is $items[0]{name}, 'Dream Theater';
}

{
    my $rs = $schema->resultset('Artist');
    my $parser = Search::Query->parser(
        default_field => [qw(id name)],
        dialect => 'DBIxClass::Fuzzy',
        query_class_opts => {
            result_source => $rs->result_source,
        },
    );
    my $query = $parser->parse("albums.name~Liquid");
    $rs = $rs->search($query->as_dbic_query);
    my @items = $rs->hashref_array;
    is @items, 1;
    is $items[0]{name}, 'Liquid Tension Experiment';
}

{
    my $rs = $schema->resultset('Artist');
    my $parser = Search::Query->parser(
        default_field => [qw(id name)],
        dialect => 'DBIxClass::Fuzzy',
        query_class_opts => {
            result_source => $rs->result_source,
        },
    );
    my $query = $parser->parse("albums.name~Liquid OR albums.name~Awake");
    $rs = $rs->search($query->as_dbic_query);
    my @items = $rs->hashref_array;
    is @items, 2;
}

{
    my $rs = $schema->resultset('Artist');
    my $parser = Search::Query->parser(
        default_field => [qw(id name)],
        dialect => 'DBIxClass::Fuzzy',
        query_class_opts => {
            result_source => $rs->result_source,
        },
    );
    my $query = $parser->parse("albums.tracks.name~Sleep");
    $rs = $rs->search($query->as_dbic_query);
    my @items = $rs->hashref_array;
    is @items, 1;
}

{
    my $code = Search::Query::Dialect::DBIxClass::Fuzzy->can('_merge_join');
    my $join = undef;

    $join = $code->($join, qw(a));
    is_deeply( $join, 'a' );

    $join = $code->($join, qw(a));
    is_deeply( $join, 'a' );

    $join = $code->($join, { a => 'b' });
    is_deeply( $join, { a => 'b' }) or diag Dumper $join;

    $join = $code->($join, { a => 'b' });
    is_deeply( $join, { a => 'b' }) or diag Dumper $join;

    $join = $code->($join, { a => 'c' });
    is_deeply( $join, { a => [qw(b c)] }) or diag Dumper $join;

    $join = $code->($join, { a => { b => 'c' } });
    is_deeply( $join, { a => [ 'c', { b => 'c' } ] } ) or diag Dumper $join;
}

done_testing;
